@@ -1,4 +1,4 @@
-# $Id: cmon.py 930 2014-06-21 15:22:48Z mwall $
+# $Id: cmon.py 1142 2014-11-25 01:00:42Z mwall $
 # Copyright 2013 Matthew Wall
 """weewx module that records cpu, memory, disk, and network usage.
 
@@ -12,19 +12,25 @@ Configuration
 Add the following to weewx.conf:
 
 [ComputerMonitor]
-    database = cmon_sqlite
+    data_binding = cmon_binding
     max_age = 2592000 # 30 days; None to store indefinitely
 
+[DataBindings]
+    [[cmon_binding]]
+        database = cmon_sqlite
+        manager = weewx.manager.DaySummaryManager
+        table_name = archive
+        schema = user.cmon.schema
+
 [Databases]
-    ...
     [[cmon_sqlite]]
         root = %(WEEWX_ROOT)s
-        database = archive/cmon.sdb
+        database_name = archive/cmon.sdb
         driver = weedb.sqlite
 
-[Engines]
-    [[WxEngine]]
-        service_list = ..., user.cmon.ComputerMonitor
+[Engine]
+    [[Services]]
+        archive_services = ..., user.cmon.ComputerMonitor
 
 
 Schema
@@ -34,32 +40,35 @@ different than the default, specify the desired schema in the configuration.
 For example, this would be a schema that stores only memory and network data,
 and uses eth1 instead of the default eth0:
 
-[ComputerMonitor]
-    database = cmon_sqlite
-    [[schema]]
-        dateTime = INTEGER NOT NULL PRIMARY KEY
-        usUnits = INTEGER
-        mem_total = INTEGER
-        mem_free = INTEGER
-        mem_used = INTEGER
-        swap_total = INTEGER
-        swap_free = INTEGER
-        swap_used = INTEGER
-        net_eth1_rbytes = INTEGER
-        net_eth1_rpackets = INTEGER
-        net_eth1_rerrs = INTEGER
-        net_eth1_rdrop = INTEGER
-        net_eth1_tbytes = INTEGER
-        net_eth1_tpackets = INTEGER
-        net_eth1_terrs = INTEGER
-        net_eth1_tdrop = INTEGER
+[DataBindings]
+    [[cmon_binding]]
+        database = cmon_sqlite
+        [[[schema]]]
+            dateTime = INTEGER NOT NULL PRIMARY KEY
+            usUnits = INTEGER NOT NULL
+            interval = INTEGER NOT NULL
+            mem_total = INTEGER
+            mem_free = INTEGER
+            mem_used = INTEGER
+            swap_total = INTEGER
+            swap_free = INTEGER
+            swap_used = INTEGER
+            net_eth1_rbytes = INTEGER
+            net_eth1_rpackets = INTEGER
+            net_eth1_rerrs = INTEGER
+            net_eth1_rdrop = INTEGER
+            net_eth1_tbytes = INTEGER
+            net_eth1_tpackets = INTEGER
+            net_eth1_terrs = INTEGER
+            net_eth1_tdrop = INTEGER
 
 Another approach to maintaining a custom schema is to define the schema in the
-file user/schemas.py as cmonSchema:
+file user/extensions.py as cmonSchema:
 
 cmonSchema = [
     ('dateTime', 'INTEGER NOT NULL PRIMARY KEY'),
-    ('usUnits', 'INTEGER'),
+    ('usUnits', 'INTEGER NOT NULL'),
+    ('interval', 'INTEGER NOT NULL'),
     ('mem_total','INTEGER'),
     ('mem_free','INTEGER'),
     ('mem_used','INTEGER'),
@@ -75,9 +84,9 @@ cmonSchema = [
 
 then load it using this configuration:
 
-[ComputerMonitor]
-    database = cmon_sqlite
-    schema = user.schemas.cmonSchema
+[DataBindings]
+    [[cmon_binding]]
+        schema = user.extensions.cmonSchema
 """
 
 # FIXME: make these methods platform-independent instead of linux-specific
@@ -94,50 +103,18 @@ from subprocess import Popen, PIPE
 
 import weewx
 import weeutil.weeutil
-from weewx.wxengine import StdService
+from weewx.engine import StdService
 
-def logmsg(level, msg):
-    syslog.syslog(level, 'cmon: %s' % msg)
+VERSION = "0.6"
 
-def logdbg(msg):
-    logmsg(syslog.LOG_DEBUG, msg)
+if weewx.__version__ < "3":
+    raise weewx.UnsupportedFeature("weewx 3 is required, found %s" %
+                                   weewx.__version__)
 
-def loginf(msg):
-    logmsg(syslog.LOG_INFO, msg)
-
-def logerr(msg):
-    logmsg(syslog.LOG_ERR, msg)
-
-def _readproc_line(filename):
-    """read single line proc file, return the string"""
-    info = ''
-    with open(filename) as fp:
-        info = fp.read()
-    return info
-
-def _readproc_lines(filename):
-    """read proc file that has 'name value' format for each line"""
-    info = {}
-    with open(filename) as fp:
-        for line in fp:
-            line = line.replace('  ',' ')
-            (label,data) = line.split(' ',1)
-            info[label] = data
-    return info
-
-def _readproc_dict(filename):
-    """read proc file that has 'name:value' format for each line"""
-    info = {}
-    with open(filename) as fp:
-        for line in fp:
-            if line.find(':') >= 0:
-                (n,v) = line.split(':',1)
-                info[n.strip()] = v.strip()
-    return info
-
-defaultSchema = [
+schema = [
     ('dateTime', 'INTEGER NOT NULL PRIMARY KEY'),
-    ('usUnits', 'INTEGER'),
+    ('usUnits', 'INTEGER NOT NULL'),
+    ('interval', 'INTEGER NOT NULL'),
     ('mem_total','INTEGER'),
     ('mem_free','INTEGER'),
     ('mem_used','INTEGER'),
@@ -243,11 +220,52 @@ NET_KEYS = [
     'tbytes','tpackets','terrs','tdrop','tfifo','tframe','tcomp','tmulti'
     ]
 
+def logmsg(level, msg):
+    syslog.syslog(level, 'cmon: %s' % msg)
+
+def logdbg(msg):
+    logmsg(syslog.LOG_DEBUG, msg)
+
+def loginf(msg):
+    logmsg(syslog.LOG_INFO, msg)
+
+def logerr(msg):
+    logmsg(syslog.LOG_ERR, msg)
+
+def _readproc_line(filename):
+    """read single line proc file, return the string"""
+    info = ''
+    with open(filename) as fp:
+        info = fp.read()
+    return info
+
+def _readproc_lines(filename):
+    """read proc file that has 'name value' format for each line"""
+    info = {}
+    with open(filename) as fp:
+        for line in fp:
+            line = line.replace('  ',' ')
+            (label,data) = line.split(' ',1)
+            info[label] = data
+    return info
+
+def _readproc_dict(filename):
+    """read proc file that has 'name:value' format for each line"""
+    info = {}
+    with open(filename) as fp:
+        for line in fp:
+            if line.find(':') >= 0:
+                (n,v) = line.split(':',1)
+                info[n.strip()] = v.strip()
+    return info
+
+
 class ComputerMonitor(StdService):
     """Collect CPU, Memory, Disk, and other computer information."""
 
     def __init__(self, engine, config_dict):
         super(ComputerMonitor, self).__init__(engine, config_dict)
+        loginf("service version is %s" % VERSION)
 
         d = config_dict.get('ComputerMonitor', {})
         self.ignored_mounts = d.get('ignored_mounts', IGNORED_MOUNTS)
@@ -257,43 +275,17 @@ class ComputerMonitor(StdService):
         self.max_age = weeutil.weeutil.to_int(d.get('max_age', 2592000))
 
         # get the database parameters we need to function
-        self.database = d['database']
-        self.table = d.get('table', 'archive')
+        binding = d.get('data_binding', 'cmon_binding')
+        self.dbm = self.engine.db_binder.get_manager(data_binding=binding,
+                                                     initialize=True)
 
-        # first look for a python path to a schema definition.  if none
-        # specified, look for name-value pairs in the configuration.  if none
-        # specified, fallback to the default schema definition.
-        schema = None
-        schema_str = d.get('schema', None)
-        if isinstance(schema_str, str):
-            logdbg("trying schema from %s" % schema_str)
-            schema = weeutil.weeutil._get_object(schema_str)
-        if schema is None and d.has_key('schema'):
-            logdbg("trying schema from configuration")
-            try:
-                dt = d['schema']['dateTime']
-                units = d['schema']['usUnits']
-                schema = []
-                for k in d['schema']:
-                    schema.append((k, d['schema'][k]))
-            except KeyError, e:
-                logdbg("schema is missing required field: %s" % e)
-                schema = None
-            except Exception, e:
-                logerr("unknown problem with schema definition: %s" % e)
-                schema = None
-        if schema is None:
-            logdbg("using default schema")
-            schema = defaultSchema
-
-        # configure the database
-        self.archive = weewx.archive.Archive.open_with_create(config_dict['Databases'][self.database], schema, self.table)
-
-        # be sure database matches the schema we have
-        dbcol = self.archive.connection.columnsOf(self.table)
-        memcol = [x[0] for x in schema]
+        # be sure schema in database matches the schema we have
+        dbcol = self.dbm.connection.columnsOf(self.dbm.table_name)
+        dbm_dict = weewx.manager.get_manager_dict(
+            config_dict['DataBindings'], config_dict['Databases'], binding)
+        memcol = [x[0] for x in dbm_dict['schema']]
         if dbcol != memcol:
-            raise Exception('schema mismatch: %s != %s' % (dbcol, memcol))
+            raise Exception('cmon schema mismatch: %s != %s' % (dbcol, memcol))
 
         # see what we are running on
         self.system = platform.system()
@@ -305,42 +297,49 @@ class ComputerMonitor(StdService):
             for key in cpuinfo:
                 loginf('cpuinfo: %s: %s' % (key, cpuinfo[key]))
 
+        self.last_ts = None
         self.last_cpu = {}
         self.last_net = {}
-        self.bind(weewx.NEW_ARCHIVE_RECORD, self.newArchiveRecordCallback)
+        self.bind(weewx.NEW_ARCHIVE_RECORD, self.new_archive_record)
 
     def shutDown(self):
-        pass
+        try:
+            self.dbm.close()
+        except:
+            pass
 
-    def newArchiveRecordCallback(self, event):
+    def new_archive_record(self, event):
         """save data to database then prune old records as needed"""
-        now = int(time.time())
+        now = int(time.time() + 0.5)
         delta = now - event.record['dateTime']
         if delta > event.record['interval'] * 60:
             logdbg("Skipping record: time difference %s too big" % delta)
             return
-        self.save_data(self.get_data())
+        if self.last_ts is not None:
+            self.save_data(self.get_data(now, self.last_ts))
+        self.last_ts = now
         if self.max_age is not None:
             self.prune_data(now - self.max_age)
 
     def save_data(self, record):
         """save data to database"""
-        self.archive.addRecord(record)
+        self.dbm.addRecord(record)
 
     def prune_data(self, ts):
         """delete records with dateTime older than ts"""
-        sql = "delete from %s where dateTime < %d" % (self.table, ts)
-        self.archive.getSql(sql)
+        sql = "delete from %s where dateTime < %d" % (self.dbm.table_name, ts)
+        self.dbm.getSql(sql)
         try:
             # sqlite databases need some help to stay small
-            self.archive.getSql('vacuum')
+            self.dbm.getSql('vacuum')
         except Exception, e:
             pass
 
-    def get_data(self):
+    def get_data(self, now_ts, last_ts):
         record = {}
-        record['dateTime'] = int(time.time()+0.5)  # required by weedb
-        record['usUnits'] = weewx.METRIC           # required by weedb
+        record['dateTime'] = now_ts                       # required
+        record['usUnits'] = weewx.METRIC                  # required
+        record['interval'] = int((now_ts - last_ts) / 60) # required
         if self.system == 'Darwin':
             record.update(self._get_macosx_info())
         elif self.system == 'BSD':
@@ -512,26 +511,32 @@ class ComputerMonitor(StdService):
 # PYTHONPATH=bin python bin/user/cmon.py
 #
 if __name__=="__main__":
-    from weewx.wxengine import StdEngine
-    config = {}
-    config['Station'] = {}
-    config['Station']['station_type'] = 'Simulator'
-    config['Station']['altitude'] = [0,'foot']
-    config['Station']['latitude'] = 0
-    config['Station']['longitude'] = 0
-    config['Simulator'] = {}
-    config['Simulator']['driver'] = 'weewx.drivers.simulator'
-    config['Simulator']['mode'] = 'simulator'
-    config['ComputerMonitor'] = {}
-    config['ComputerMonitor']['database'] = 'cmon_sqlite'
-    config['Databases'] = {}
-    config['Databases']['cmon_sqlite'] = {}
-    config['Databases']['cmon_sqlite']['root'] = '%(WEEWX_ROOT)s'
-    config['Databases']['cmon_sqlite']['database'] = '/tmp/cmon.sdb'
-    config['Databases']['cmon_sqlite']['driver'] = 'weedb.sqlite'
-    config['Engines'] = {}
-    config['Engines']['WxEngine'] = {}
-    config['Engines']['WxEngine']['service_list'] = 'user.cmon.ComputerMonitor'
+    from weewx.engine import StdEngine
+    config = {
+        'Station': {
+            'station_type': 'Simulator',
+            'altitude': [0, 'foot'],
+            'latitude': 0,
+            'longitude': 0},
+        'Simulator': {
+            'driver': 'weewx.drivers.simulator',
+            'mode': 'simulator'},
+        'ComputerMonitor': {
+            'binding': 'cmon_binding'},
+        'DataBindings': {
+            'cmon_binding': {
+                'database': 'cmon_sqlite',
+                'manager': 'weewx.manager.DaySummaryManager',
+                'table_name': 'archive',
+                'schema': 'user.cmon.schema'}},
+        'Databases': {
+            'cmon_sqlite': {
+                'root': '%(WEEWX_ROOT)s',
+                'database_name': '/tmp/cmon.sdb',
+                'driver': 'weedb.sqlite'}},
+        'Engine': {
+            'Services': {
+                'archive_services': 'user.cmon.ComputerMonitor'}}}
     engine = StdEngine(config)
     svc = ComputerMonitor(engine, config)
     record = svc.get_data()
