@@ -1,4 +1,4 @@
-# $Id: install.py 930 2014-06-21 15:22:48Z mwall $
+# $Id: install.py 1142 2014-11-25 01:00:42Z mwall $
 # installer for cmon
 # Copyright 2014 Matthew Wall
 
@@ -10,7 +10,7 @@ def loader():
 class ComputerMonitorInstaller(ExtensionInstaller):
     def __init__(self):
         super(ComputerMonitorInstaller, self).__init__(
-            version="0.5",
+            version="0.6",
             name='cmon',
             description='Collect and display computer health indicators.',
             author="Matthew Wall",
@@ -18,16 +18,21 @@ class ComputerMonitorInstaller(ExtensionInstaller):
             process_services='user.cmon.ComputerMonitor',
             config={
                 'ComputerMonitor': {
-                    'database': 'cmon_sqlite',
-                    'max_age': '2592000'},
+                    'data_binding': 'cmon_binding'},
+                'DataBindings': {
+                    'cmon_binding': {
+                        'database': 'cmon_sqlite',
+                        'table_name': 'archive',
+                        'manager': 'weewx.manager.DaySummaryManager',
+                        'schema': 'user.cmon.schema'}},
                 'Databases': {
                     'cmon_sqlite': {
-                        'database': 'cmon.sdb',
+                        'database_name': 'cmon.sdb',
                         'driver': 'weedb.sqlite'}},
                 'StdReport': {
                     'cmon': {
-                        'skin':'cmon',
-                        'HTML_ROOT':'cmon' }}},
+                        'skin': 'cmon',
+                        'HTML_ROOT': 'cmon'}}},
             files=[('bin/user',
                     ['bin/user/cmon.py']),
                    ('skins/cmon',
